__version__ = "0.2.6"
_shiny_output_class = "shiny-maplibregl-output"

# TODO: Remove from here
# tmp
_points = "https://raw.githubusercontent.com/visgl/deck.gl-data/master/examples/scatterplot/manhattan.json"
_geojson = "https://docs.mapbox.com/mapbox-gl-js/assets/indoor-3d-map.geojson"
_urban_areas = "https://d2ad6b4ur7yvpq.cloudfront.net/naturalearth-3.3.0/ne_50m_urban_areas.geojson"
_highway_lines = (
    "https://github.com/visgl/deck.gl-data/raw/master/examples/highway/roads.json"
)

_airports = (
    "https://github.com/visgl/deck.gl-data/raw/master/examples/line/airports.json"
)
_flights = "https://github.com/visgl/deck.gl-data/raw/master/examples/line/heathrow-flights.json"
